package main

import (
	"bufio"
	"errors"
	"fmt"
	"io"
	"log"
	"os"
	"slices"
)

const MAX_LOOP = 1_000_000

func trim(l []byte) []byte {
	if len(l) > 0 && l[len(l)-1] == '\n' {
		return l[:len(l)-1]
	}
	return l
}

func main() {
	topMap := make([]int, 0, 256)
	cursors := make([]Cursor, 0, 32)
	trailheads := make([]int, 0, 8)

	// Read input
	r := bufio.NewReader(os.Stdin)

	width, topMap, trailheads, err :=
		readMap(topMap, trailheads, r)
	if err != nil {
		log.Fatalf("While reading map input: %v", err)
	}
	height := len(topMap) / width

	// Start at trailheads
	cursors = cursors[:len(trailheads)]

	for i, t := range trailheads {
		cursors[i].start = t
		cursors[i].idx = t
	}

	defunct := 0
	for defunct != len(cursors) {
		for i, c := range cursors {
			idx := c.idx
			if idx < 0 {
				continue
			}
			nextHeight := topMap[idx] + 1
			if nextHeight == 10 {
				cursors[i].idx = -2
				cursors[i].end = idx
				defunct++
				continue
			}

			viable := 0
			ns := neighbors(idx, width, height)
			for _, n := range ns {
				if topMap[n] != nextHeight {
					continue
				}
				viable++
				if viable == 1 {
					cursors[i].idx = n
					continue
				}
				l := len(cursors)
				if l == cap(cursors) {
					cursors = append(cursors, Cursor{})
				}
				cursors = cursors[:l+1]
				cursors[l].start = c.start
				cursors[l].idx = n
			}
			// If we encountered no viable neighbors, cursor is
			// defunct.
			if cursors[i].idx == idx {
				cursors[i].idx = -1
				defunct++
			}
		}
	}

	// Determine scores, and output
	sum := 0
	paths := make(map[Cursor]struct{})
	scores := make([]int, len(trailheads))
	for _, c := range cursors {
		switch c.idx {
		case -1:
			continue
		case -2:
			paths[c] = struct{}{}
		default:
			log.Fatalf("BUG: unexpected cursor index in %#v", c)
		}
	}
	for p := range paths {
		sum++
		i := slices.IndexFunc(trailheads, func(e int) bool {
			return e == p.start
		})
		scores[i]++
	}
	for i, s := range scores {
		coord := CoordOf(trailheads[i], width, height)
		fmt.Printf("(%d, %d): %d\n", coord.x, coord.y, s)
	}
	fmt.Printf("sum: %d\n", sum)
}

type Cursor struct{ start, end, idx int }

type Coord struct{ x, y uint }

func CoordOf(i, w, h int) (c Coord) {
	c.y = uint(i) / uint(w)
	c.x = uint(i) % uint(w)
	return c
}

func indexOf(c Coord, w int) int {
	return int(c.y*uint(w) + c.x)
}

func neighbors(i, w, h int) []int {
	c := CoordOf(i, w, h)
	candidates := []Coord{
		{c.x + 1, c.y},
		{c.x - 1, c.y},
		{c.x, c.y + 1},
		{c.x, c.y - 1},
	}
	ns := make([]int, 0, 4)
	for _, n := range candidates {
		if n.x < uint(w) && n.y < uint(h) {
			idx := indexOf(n, w)
			if idx < 0 {
				continue
			}
			ns = append(ns, indexOf(n, w))
		}
	}
	return ns
}

func readMap(
	topMap []int,
	trailheads []int,
	r *bufio.Reader,
) (
	width int,
	nextTopMap []int,
	nextTrailheads []int,
	err error,
) {
	line := 0
	for i := 0; i < MAX_LOOP; i++ {
		b, err := r.ReadByte()
		if errors.Is(err, io.EOF) {
			break
		}
		if err != nil {
			log.Fatalf("While reading input: %v", err)
		}
		topMap = append(topMap, int(b-48))

		switch {
		case b == '0':
			trailheads = append(trailheads, len(topMap)-1)
		case '1' <= b && b <= '9':
		case b == '.':
			topMap[len(topMap)-1] = -1
		case b == '\n':
			if width == 0 {
				width = i
			}
			line++
			topMap = topMap[:len(topMap)-1]

		default:
			col := i + 1
			if width != 0 {
				col %= width
			}
			log.Fatalf("Unexpected input '%q' at line %d, column %d", b, line, col)
		}
	}
	nextTrailheads = trailheads
	nextTopMap = topMap
	return
}
